(*
 *  beware IRC services, bservicemngr.pas
 *  Copyright (C) 2002 Bas Steendijk
 *
 *  some modifications by peter green
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
unit bservicemngr;

{
it must be possible to start the bot at any time by calling its onstartup

it then should be possible later to unload the bot, calling it's shutdown procedure

the bot should be in the same, stable, state, before startup, and after shutdown.
}


interface

uses classes,buser,bstuff,bconsts,pgtypes,bchannel;


type
  tonprivmsg=procedure(nickatserver:boolean;sptr:tuser;target:tuser;s:string;tag:taddrint);
  tonchanmsg=procedure(sptr:tuser;target:tchannel;s:string;tag:taddrint);
  tonaccount=procedure(sptr:tuser;us:tuser;tag:taddrint);
  tonnewuser=procedure(us:tuser;tag:taddrint);
  tonsignal=procedure(s:string;tag:taddrint);
  tonleavechannel=procedure(channel:tchannel;source:tuser;target:tuser;kick:boolean;message:string;tag:taddrint);
  toninvite=procedure(sptr:tuser;user:tuser;channel:tchannel;tag:taddrint);
  {tonparse=procedure(sptr:tuser;parc:integer;parv:pparams;tag:taddrint);}
  tnotify=procedure(tag:taddrint);

  tserviceunit=class
    name:string;
    num:integer;
    onsendburst:tnotify;
    ondisconnect:tnotify;
    onstartup:tnotify;
    onshutdown:tnotify;
    onsave:tnotify;
    onprivmsg:tonprivmsg;
    onchanmsg:tonchanmsg;
    onnewuser:tonnewuser;
    ondestroyuser:tonnewuser;
    ontimer:tnotify;
    onrehash:tnotify;
    onleavechannel:tonleavechannel;
    onaccount:tonaccount;
    oninvite:toninvite;
    {onparse:tonparse;}

    tag:taddrint;
  end;

function createserviceunit:tserviceunit;
function registerserviceunit(t:tserviceunit):boolean;
function addbot(srvunit:tserviceunit;nick,userid,host,info:string;modes:integer):tuser;
function srvunitisonchannel(srvunit:tserviceunit;ch:tchannel):boolean;

var
  serviceunitarray:array[0..maxserviceunit] of tserviceunit;
  botarray:array[0..maxbot] of pointer;

implementation

uses bcrash,bconfig,bparse,bservunit,bcmds,blinklist;

function createserviceunit:tserviceunit;
begin
  result := tserviceunit.create;
end;

function registerserviceunit(t:tserviceunit):boolean;
var
  a:integer;
begin
  {find free slot in service unit array}
  result := false;
  for a := 0 to maxserviceunit do begin
    if serviceunitarray[a] = nil then begin
      serviceunitarray[a] := t;
      t.num := a;
      result := true;
      exit;
    end;
  end;
end;

function addbot(srvunit:tserviceunit;nick,userid,host,info:string;modes:integer):tuser;
var
  a,b:integer;
  us:tuser;
begin
  {find free bot slot}
  b := -1;
  for a := 0 to maxbot do begin
    if me.server.p10slots[a] = nil then begin
      b := a;
      break;
    end;
  end;
  assert(b >= 0,'addbot: no free slots','');
  assert(nick <> '','addbot: no nick given','');

  {prevent nick collision. the nick is mine.}
  us := findname(nick);
  if us <> nil then begin
    assert(us.server <> me.server,'Addbot: nick collision with other service: '+nick,''); {addbot fails, another service bot has the nick}

    {send kill}
    sendkill(us,me.name,'Don''t use this nick');
  end;

  us := adduser;
  setnickname(us,nick);
  us.ts := globaldefaultts;
  if userid = '' then
  us.userid := opt.botuserid
  else us.userid := userid;

  assert(us.userid <> '','addbot: no userid given','');

  if host = '' then
  us.host := opt.bothost
  else us.host := host;

  assert(us.host <> '','addbot: no host given','');

  us.uhost := ircupper(us.host);

  us.binip := botip;

  us.modeflag := modes;

  setflag(us.flags,userflag_myuser or userflag_isclient);
  us.idstr := mynum+p10inttostr(b,CCClen);

  us.server := me.server;
  us.p10slotnum := b;
  me.server.p10slots[b] := us;
  if info = '' then
  us.fullname := opt.info
  else
  us.fullname := info;
  botarray[b] := us;
  us.srvunit := srvunit;
  result := us;
  inc(count.globalclients);
  if flag_isset(us.modeflag,usermode_invisible) then inc(count.invisible);
  if isoper(us) then inc(count.oper);
  if online then sendbot_nick(us);
  onnewuser(us);
end;

function srvunitisonchannel(srvunit:tserviceunit;ch:tchannel):boolean;
var
  p2:tlinklist2;
begin
  result := false;
  p2 := ch.user;
  while p2 <> nil do begin
    if tserviceunit(tuserchan(p2).us.srvunit) = srvunit then begin
      result := true;
      exit;
    end;
    p2 := p2.next2;
  end;
end;
begin
  fillchar(serviceunitarray,sizeof(serviceunitarray),0);
end.
